/*
 * tmos - pommax2.c
 * 	version 0.1.0
 * 
 * Copyright (C) 2009..2012  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2009  Marek Piwonski <marek@microster.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/console.h>
#include <tmos/adc.h>
#include <tmos/dac.h>
#include <tmos/watchdog.h>
#include <tmos/reboot.h>
#include "pommax2.h"

u8 pommax2_lowpass_enable;
u8 pommax2_rms_enable;

#ifdef CONFIG_POMMAX2_SER_REBOOT
static unsigned char reboot_n;

static void reboot_check(void) __reentrant
{
	char c;

	c = ser_rxc();
	if (c == 'x') {
		reboot_n++;
		if (reboot_n >= 5)
			machine_restart();
	} else {
		reboot_n = 0;
	}
}
#endif

void pommax2_offset_configure(void)
{
	int16_t x;

	x = pommax2_config_read(POMMAX2_REG_OFFSET);
	dac_write(0, x);
	adc_set_offset(x);
}

void pommax2_filter_configure(void)
{
	u8 flags = pommax2_config_read(POMMAX2_REG_FLAGS);
	u8 rms = 0;
	u8 raw = 0;

	if (flags & POMMAX3_FLAGS_FILTER) {
		rms = pommax2_config_read(POMMAX2_REG_RMS);
		raw = pommax2_config_read(POMMAX2_REG_RAW);
	}

	pommax2_rms_enable = rms & ~raw;
	pommax2_lowpass_enable = ~(rms | raw);
}

static void pommax_configure(void)
{
	u16 x;

	pommax2_config_load();

	x = pommax2_config_read(POMMAX2_REG_FREQ);
	if (x)
		adc_set_freq(x);
	pommax2_gain_configure();
	watchdog_refresh();
	pommax2_offset_configure();
	pommax2_filter_configure();
	pommax2_proto_configure();
}

int app_main(void) REENTRANT
{
	uint8_t seq = 0;

	watchdog_refresh();
	pommax_configure();
	for (;;) {
		watchdog_refresh();

		seq = adc_sync(seq);
		pommax2_send_sample();

#ifdef CONFIG_POMMAX2_SER_REBOOT
		if (ser_rx_poll())
			reboot_check();
#endif
	}
}
