/*
 * spruked - mach-birdie.c
 *
 * Machine-dependend code for BirDie board
 *
 * Copyright (C) 2013  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <tmos/kernel.h>
#include <tmos/serial.h>
#include <tmos/bitrev.h>
#include <tmos/spruked.h>

#include "spruked.h"

#define IO_0	P2_6
#define IO_1	P2_7
#define IO_2	P1_4

#define PORT_LO		P0
#define PORT_HI		P2

static unsigned char addr;

static u8 spruked_get_addr(void)
{
	unsigned char ret = 0;

	if (!IO_0)
		ret |= 8;
	if (!IO_1)
		ret |= 4;
	if (!IO_2)
		ret |= 2;
	return ret;
}

void spruked_mach_poll(void)
{
	struct spruked_skb __kernel *skb;
	struct spruked_skb __kernel *skb2;
	u8 __kernel *p;
	u8 p0;
	u8 p1;
	u8 p2;
	u8 p3;

	skb = spruked_alloc_skb();
	if (!skb)
		return;
	skb2 = spruked_alloc_skb();
	if (!skb2) {
		spruked_kfree_skb(skb);
		return;
	}

	/* read all inputs with smallest possible time difference */
	p0 = P0;
	p1 = P1;
	p2 = P2;
	p3 = P3;

	p = skb->payload;
	*p++ = addr << 4;

	/*
	 * pin remapping - this is evil, but the PCB with such mapping
	 * is much better
	 */
	p2 <<= 2;
	p2 |= p3 >> 6;
	*p++ = 0;
	*p++ = ~p2;

	p = skb2->payload;
	*p++ = (addr + 1) << 4;

	p0 = bitrev8(p0);
	*p++ = ~p0;
	p1 &= 0x0f;
	p3 <<= 2;
	p1 |= p3 & 0xf0;
	*p++ = ~p1;

	spruked_xmit(skb);
	spruked_xmit(skb2);
}

static void birdie_init(void)
{
	P0 = 0xff;
	P1 = 0xff;
	P2 = 0xff;
	P3 = 0xff;

	addr = spruked_get_addr();
}
late_initcall(birdie_init)
