/*
 * spruked.c
 *
 * Copyright (C) 2013  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2013  Karol Budniak <karol@microster.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <tmos/kernel.h>
#include <tmos/spruked.h>
#include <tmos/sched.h>
#include <tmos/crc7k.h>
#include <tmos/watchdog.h>

#include "spruked.h"

/*
 * We send data with SPRUKED_BUS_CYCLE period. Note that the MS_JIFFIES
 * converts miliseconds to jiffies with rouding up and this conversion
 * might increase period. For 1 ms period HZ=1000 is needed.
 */
#define SPRUKED_BUS_CYCLE	MS_JIFFIES(1)

unsigned char spruked_bus_cycle = SPRUKED_BUS_CYCLE;

#ifdef CONFIG_SPRUKED_STATS
static void update_link_state(void)
{
	static unsigned char old_rx_packets;
	struct spruked_stats stats;
	unsigned char rx_packets;
	unsigned char link_ok;

	if (spruked_get_stats(&stats))
		return;
	rx_packets = stats.rx_packets;
	link_ok = old_rx_packets != rx_packets;
	old_rx_packets = rx_packets;

	spruked_mach_link_state(link_ok);
}
#else
#define update_link_state()	do { } while (0)
#endif

static jiffies_t link_state_next;

int app_main(void) __reentrant
{
	jiffies_t t;

	t = get_jiffies();
	link_state_next = t;
	for (;;) {
		watchdog_refresh();
		spruked_mach_poll();
		if (time_after(t, link_state_next)) {
			link_state_next += MS_JIFFIES(80);
			update_link_state();
		}

		/*
		 * wait for next cycle
		 *
		 * this code assumes that we are faster than configured
		 * SPRUKED_BUS_CYCLE and we will wait here. Otherwise
		 * the time difference between current jiffies and
		 * expected wake-up jiffy t will increase and overflow.
		 */
		t += spruked_bus_cycle;
		while (!time_after_eq(get_jiffies(), t)) {
			watchdog_refresh();
			schedule();
		}
	}
}
