/*
 * wysmod - main.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2005..2009  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2005..2008  Marek Piwoski <marek@microster.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/console.h>
#include <tmos/watchdog.h>
#include <tmos/jiffies.h>
#include <tmos/serial.h>
#include <tmos/port.h>
#include "display.h"
#include "ar2clp.h"

#ifndef dir_tx
#define dir_tx()	do { } while (0)
#endif
#ifndef dir_rx
#define dir_rx()	do { } while (0)
#endif

char wysmod_fd;

static void unsupported_mode(void)
{
	static const __icode uint8_t scr[] =
		WYSMOD_VERSION
		"  Unsupported mode  ";

	lcd_put(scr);
}

static void init_sleep(void)
{
	unsigned char i;

	for (i = 100; i > 0; i--) {
		watchdog_refresh();
		msleep(10);
	}
}

static const char clear_line[] = "                    ";

static void splash_screen(unsigned char mode)
{
	static const char *const modes[] = {
		"WYSMOD 38.4k",
		"WYDAL 9.6k",
		"WYSREJ 9.6k",
		"AUTODETECT",
	};
	const char *p = "unknown";

	if (mode < ARRAY_SIZE(modes))
		p = modes[mode];
	c_puts_p(c_mkpos(1, 0), clear_line);
	c_puts_p(c_mkpos(1, 0), "MODE: ");
	c_puts_p(c_mkpos(1, 6), p);
	c_flush();
	init_sleep();
}

#if CONFIG_WYSMOD_WYDAL || CONFIG_WYSMOD_WYSREJ
static void init_serial(void)
{
	wysmod_fd = port_open("/dev/ttyS0", O_RDWR);
	if (wysmod_fd == -1)
		for (;;);
}

static unsigned char detect_mode(void)
{
	u8 buf[8];
	char ret;

	dir_rx();
	ser_set_spd(BDEFAULT);
	c_puts_p(c_mkpos(0, 0), WYSMOD_VERSION);
	c_puts_p(c_mkpos(1, 0), clear_line);

	/*
	 * if we're receiving something assume for now that it's Modbus RTU
	 * interface.
	 */
	ret = port_read_timeout(wysmod_fd, buf, sizeof(buf), MS_JIFFIES(500));
	if (ret > 4)
		return MODE_WYSMOD;

	ser_set_spd(B9600);
	dir_tx();
	tmos_memcpy(buf, get_id_cmd, sizeof(get_id_cmd));
	port_write(wysmod_fd, buf, sizeof(get_id_cmd));
	dir_rx();

	/*
	 * If response was received we assume that it's AR-3c and all
	 * AR-3c's support WYDAL protocol
	 */
	ret = port_read_timeout(wysmod_fd, buf, sizeof(buf), MS_JIFFIES(200));
	if (ret == 2)
		return MODE_WYDAL;

	return MODE_WYSREJ;
}
#else
#define init_serial() do { } while (0)
#define detect_mode() (MODE_WYSMOD)
#endif

int app_main(void)
{
	unsigned char mode;

	init_serial();

retry:
	c_puts_p(c_mkpos(0, 0), WYSMOD_VERSION);

       	mode = MODE;
	splash_screen(mode);
	if (mode == MODE_AUTO) {
		mode = detect_mode();
		splash_screen(mode);
	}

	switch (mode) {
#ifdef CONFIG_WYSMOD_WYSMOD
	case MODE_WYSMOD:
		wysmod();
		break;
#endif
#ifdef CONFIG_WYSMOD_WYDAL
	case MODE_WYDAL:
		ser_set_spd(B9600);
		wydal();
		break;
#endif
#ifdef CONFIG_WYSMOD_WYSREJ
	case MODE_WYSREJ:
		ser_set_spd(B9600);
		wysrej();
		break;
#endif
	default:
		unsupported_mode();
	}
	goto retry;
}
