/*
 * tmos - ipage.c
 *
 * IRAM paged memory allocator
 *
 * Copyright (C) 2013  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <tmos/kernel.h>
#include <tmos/ipage.h>

/* reserve some memory for stack */
#define STACK_RESERVE		48

static u8 __kernel *ipage_next;

/**
 * ipage_alloc - allocate IRAM page
 *
 * This function can be called only in kernel context.
 */
void __kernel *ipage_alloc(void)
{
	u8 __kernel *ret;

	preempt_disable();
	ret = ipage_next;
	if (ret)
		ipage_next = (u8 __kernel *) *ret;
	preempt_enable();
	return ret;
}

/**
 * ipage_alloc - free IRAM page
 * @p: page to free
 *
 * This function can be called only in kernel context.
 */
void ipage_free(void __kernel *p)
{
	u8 __kernel *i = p;

	preempt_disable();
	*i = (unsigned char) ipage_next;
	ipage_next = i;
	preempt_enable();
}

void ipage_init(void)
{
	unsigned char p;

	p = ((SP + STACK_RESERVE + IPAGE_SIZE - 1) & ~(IPAGE_SIZE - 1));

	if (p >= SP) {
		u8 __kernel *i = (u8 __kernel *) p;
		do {
			*i = (unsigned char) ipage_next;
			ipage_next = i;
			i += IPAGE_SIZE;
		} while (i);
	}
}
arch_initcall(ipage_init)
