/*
 * tmos - udelay.c
 *
 * Implementation of udelay() function
 * 
 * Copyright (C) 2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/config.h>
#include <tmos/jiffies.h>
#include <asm/delay.h>

/**
 * udelay - wait @n microseconds
 * @n: minimal number microseconds to wait, < 10000
 *
 * This function returns after at least @n microseconds. The number
 * of microseconds must be smaller than 10000. Otherwise overflow
 * may occur.
 */
void __udelay(unsigned int n)
{
	unsigned int loops;

	loops = (unsigned long) n * loops_per_jiffy / (1000000 / HZ);
	loops++;
	__delay(loops);
}
