/*
 * rms.h
 *
 * Root Mean Square filter
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

static uint16_t r_y1;
static uint16_t r_y2;
static uint16_t r_y3;
static uint8_t r_e1;
static uint8_t r_e2;

FILTER_CODE
{
	static __bit phase;
	register int t;
	register unsigned char x8;

	if (x == -2048)
		x = 2047;
	else if (x < 0)
		x = -x;
	x8 = x >> 3;

	t = ((unsigned int)(x8 * x8 + 1) >> 1) - r_y1 + r_e1;
	r_y1 += t >> 7;
	r_e1 = t & 0x7f;

	if (phase) {
		t = r_y1 - r_y2 + r_e2;
		r_y2 += t >> 7;
		r_e2 = t & 0x7f;
	} else {
		x8 = (unsigned int) r_y3 >> 8;
		t = (int)r_y2 - (int)((unsigned int)(x8 * x8 + 1) >> 1);
		r_y3 += t;
		if (t >= 0 && r_y3 < (unsigned int)t)
			r_y3 = 65535;
	}
	phase = !phase;
}
