/*
 * tmos - spruked.h
 *
 * spruked networking support
 *
 * Copyright (C) 2013  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef __TMOS_SPRUKED_H
#define __TMOS_SPRUKED_H

#include <tmos/types.h>
#include <tmos/kernel.h>
#include <asm/softirq.h>

/*
 * initial checksum - we start from 0x7f to avoid CRC problems with
 * inserting zeros before checksum - in this application it's not needed
 * but it's a common practice.
 */
#define SPRUKED_CHECKSUM_INIT		0x7f
#define SPRUKED_CHECKSUM_CHECK		0x37

struct spruked_skb {
	union {
		struct {
			struct spruked_skb __kernel *next;
		};
		u8 payload[4];
	};
};

extern struct spruked_skb __kernel *skbs_free;
extern struct spruked_skb __kernel *spruked_rx_skb;

void spruked_softirq(void) __using(SOFTIRQ_BANK);

#ifdef CONFIG_SPRUKED_PROTO
#define call_spruked_softirq() \
	do { \
		if (spruked_softirq_pending) { \
			spruked_softirq_pending = 0; \
			spruked_softirq(); \
		} \
	} while (0);
#else
#define call_spruked_softirq() do { } while (0)
#endif

extern __bit spruked_tx_pending;

extern __bit spruked_softirq_pending;
#define spruked_pend_softirq() \
	do { \
		spruked_softirq_pending = 1; \
		raise_softirq(); \
	} while (0)

#define spruked_received_frame() \
	do { \
		spruked_pend_softirq(); \
	} while (0);

#define spruked_tx_completed() spruked_pend_softirq()

#define __spruked_consume_skb(skb) \
	do { \
		(skb)->next = skbs_free; \
		skbs_free = (skb); \
	} while (0)
#define __spruked_kfree_skb(skb) __spruked_consume_skb(skb)

void spruked_consume_skb(struct spruked_skb __kernel *skb);
#define spruked_kfree_skb(skb) spruked_consume_skb(skb)

struct spruked_skb __kernel *spruked_alloc_skb(void);

#define __spruked_alloc_skb(skb) \
	do { \
		(skb) = skbs_free; \
		if ((skb)) \
			skbs_free = skb->next; \
	} while (0)

#define __spruked_alloc_skb_poll(skb) \
	do { \
		(skb) = skbs_free; \
	} while (0)

#define __spruked_alloc_skb_commit(skb) \
	do { \
		skbs_free = skb->next; \
	} while (0)

void spruked_xmit(struct spruked_skb __kernel *skb);
void spruked_xmit_start(struct spruked_skb __kernel *skb) __using(SOFTIRQ_BANK);

struct spruked_stats {
	unsigned char rx_packets;
	unsigned char rx_crc_errors;
};

#if CONFIG_SPRUKED_STATS
extern char spruked_get_stats(struct spruked_stats __kernel *stats);
#else
static inline char spruked_get_stats(struct spruked_stats __kernel *stats)
{
	stats->rx_packets = 0;
	stats->rx_crc_errors = 0;
	return 0;
}
#endif

void spruked_skb_init(void);

#endif
