/*
 * tmos - net.c
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <tmos/kernel.h>
#include <asm/8051.h>
#include <tmos/net.h>

/*
 * net_softirq - poll all network interfaces
 */
void net_softirq(void) __reentrant
{
#define NET_IF(name, poll, ioctl) \
	poll();
#include <tmos/net-ifs.h>
#undef NET_IF
}

#ifdef NET_INTERFACE_IDS
char __net_ioctl(unsigned char netif, unsigned char cmd, void __kernel *arg)
	__reentrant
{
#error "multiple interfaces not supported yet"
}
#else
char __net_ioctl(unsigned char cmd, void __kernel *arg) __reentrant
{
#define NET_IF(name, poll, ioctl) \
	return ioctl(cmd, arg);
#include <tmos/net-ifs.h>
#undef NET_IF
}
#endif

net_size_t net_xmit(net_proto_t proto, struct net_msg __kernel *msg) __reentrant
{
	switch (proto) {
#ifdef CONFIG_MB_SLAVE_NET
	case NET_PROTO_MODBUS:
		return modbus_xmit(msg);
#endif
	}
	/* just ignore unknown protocols */
	return 0;
}
